@@ -35,6 +35,7 @@ struct sip_request {
 	sip_send_h *sendh;
 	sip_resp_h *resph;
 	void *arg;
+	size_t sortkey;
 	enum sip_transp tp;
 	bool tp_selected;
 	bool stateful;
@@ -227,6 +228,7 @@ static int request_next(struct sip_request *req)
 		list_unlink(&rr->le);
 
 		if (req->addrl.head) {
+			dns_rrlist_sort_addr(&req->addrl, req->sortkey);
 			mem_deref(rr);
 			goto again;
 		}
@@ -259,6 +261,11 @@ static int request_next(struct sip_request *req)
 		if (req->addrl.head || req->srvl.head)
 			goto again;
 	}
+	else if (!req->stateful) {
+		req->resph = NULL;
+		terminate(req, 0, NULL);
+		mem_deref(req);
+	}
 
 	return err;
 }
@@ -387,7 +394,7 @@ static void naptr_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 	(void)hdr;
 	(void)authl;
 
-	dns_rrlist_sort(ansl, DNS_TYPE_NAPTR);
+	dns_rrlist_sort(ansl, DNS_TYPE_NAPTR, req->sortkey);
 
 	rr = dns_rrlist_apply(ansl, NULL, DNS_TYPE_NAPTR, DNS_CLASS_IN, false,
 			      rr_naptr_handler, req);
@@ -405,8 +412,6 @@ static void naptr_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 		return;
 	}
 
-	dns_rrlist_sort(addl, DNS_TYPE_SRV);
-
 	dns_rrlist_apply(addl, rr->rdata.naptr.replace, DNS_TYPE_SRV,
 			 DNS_CLASS_IN, true, rr_append_handler, &req->srvl);
 
@@ -420,6 +425,8 @@ static void naptr_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 		return;
 	}
 
+	dns_rrlist_sort(&req->srvl, DNS_TYPE_SRV, req->sortkey);
+
 	dns_rrlist_apply(addl, NULL, DNS_QTYPE_ANY, DNS_CLASS_IN, false,
 			 rr_cache_handler, req);
 
@@ -427,12 +434,6 @@ static void naptr_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 	if (err)
 		goto fail;
 
-	if (!req->stateful) {
-		req->resph = NULL;
-		terminate(req, 0, NULL);
-		mem_deref(req);
-	}
-
 	return;
 
  fail:
@@ -448,8 +449,6 @@ static void srv_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 	(void)hdr;
 	(void)authl;
 
-	dns_rrlist_sort(ansl, DNS_TYPE_SRV);
-
 	dns_rrlist_apply(ansl, NULL, DNS_TYPE_SRV, DNS_CLASS_IN, false,
 			 rr_append_handler, &req->srvl);
 
@@ -480,6 +479,8 @@ static void srv_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 		return;
 	}
 
+	dns_rrlist_sort(&req->srvl, DNS_TYPE_SRV, req->sortkey);
+
 	dns_rrlist_apply(addl, NULL, DNS_QTYPE_ANY, DNS_CLASS_IN, false,
 			 rr_cache_handler, req);
 
@@ -487,12 +488,6 @@ static void srv_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 	if (err)
 		goto fail;
 
-	if (!req->stateful) {
-		req->resph = NULL;
-		terminate(req, 0, NULL);
-		mem_deref(req);
-	}
-
 	return;
 
  fail:
@@ -521,16 +516,12 @@ static void addr_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 		goto fail;
 	}
 
+	dns_rrlist_sort_addr(&req->addrl, req->sortkey);
+
 	err = request_next(req);
 	if (err)
 		goto fail;
 
-	if (!req->stateful) {
-		req->resph = NULL;
-		terminate(req, 0, NULL);
-		mem_deref(req);
-	}
-
 	return;
 
  fail:
@@ -595,6 +586,7 @@ static int addr_lookup(struct sip_request *req, const char *name)
  * @param uril     Length of Request URI string
  * @param route    Next hop route URI
  * @param mb       Buffer containing SIP request
+ * @param sortkey  Key for DNS record sorting
  * @param sendh    Send handler
  * @param resph    Response handler
  * @param arg      Handler argument
@@ -603,7 +595,7 @@ static int addr_lookup(struct sip_request *req, const char *name)
  */
 int sip_request(struct sip_request **reqp, struct sip *sip, bool stateful,
 		const char *met, int metl, const char *uri, int uril,
-		const struct uri *route, struct mbuf *mb,
+		const struct uri *route, struct mbuf *mb, size_t sortkey,
 		sip_send_h *sendh, sip_resp_h *resph, void *arg)
 {
 	struct sip_request *req;
@@ -639,6 +631,7 @@ int sip_request(struct sip_request **reqp, struct sip *sip, bool stateful,
 		goto out;
 
 	req->stateful = stateful;
+	req->sortkey = sortkey;
 	req->mb    = mem_ref(mb);
 	req->sip   = sip;
 	req->sendh = sendh;
@@ -775,7 +768,7 @@ int sip_requestf(struct sip_request **reqp, struct sip *sip, bool stateful,
 	mb->pos = 0;
 
 	err = sip_request(reqp, sip, stateful, met, -1, uri, -1, route, mb,
-			  sendh, resph, arg);
+			  (size_t)arg, sendh, resph, arg);
 	if (err)
 		goto out;
 
@@ -842,7 +835,8 @@ int sip_drequestf(struct sip_request **reqp, struct sip *sip, bool stateful,
 	mb->pos = 0;
 
 	err = sip_request(reqp, sip, stateful, met, -1, sip_dialog_uri(dlg),
-			  -1, sip_dialog_route(dlg), mb, sendh, resph, arg);
+			  -1, sip_dialog_route(dlg), mb, sip_dialog_hash(dlg),
+			  sendh, resph, arg);
 	if (err)
 		goto out;
 
